/**
 * @file LED_Flash_click_AVR.c
 * @brief <h2> Blitz Click Board Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * LED Flash click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        09/11/2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    ATMEGA32
 * -<b> Dev. Board    </b> :    EasyAVR v7
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    LED Flash Click Board
 * -<b> SW            </b> :    MikroC PRO for AVR v6.1.2
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put LED Flash click in mikroBUS 1 slot.
 * 2) Button press level for PA set to GND.
 * 3) Set pull-up for PA0 and PA1.
 */

/**************************************************************************
* Includes
**************************************************************************/
#include <stdint.h>

/**************************************************************************
* Module Preprocessor Constants
**************************************************************************/
#define POLLING_PERIOD 10

sbit FLASH      at PORTA6_bit;
sbit FLASH_DIR  at DDA6_bit;

sbit TORCH      at PORTD4_bit;
sbit TORCH_DIR  at DDD4_bit;

sbit CHRG       at PORTA7_bit;
sbit CHRG_DIR   at DDA7_bit;

sbit FLAG       at PIND2_bit;
sbit FLAG_DIR   at DDD2_bit;

/**************************************************************************
* Module Typedefs
**************************************************************************/
typedef enum state 
{
    STATE_READY,
    STATE_CHARGE,
    STATE_TORCH,
    STATE_FLASH
} State;

/**************************************************************************
* Module Variable Definitions
**************************************************************************/
// LCD module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

uint8_t buttonstate_a0 = 0xFF;
uint8_t buttonstate_a1 = 0xFF;

State state;
State next_state;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void MCU_Init();
uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state);

/******************************************************************************
* Public Functions
*******************************************************************************/
void main() 
{
    MCU_Init();

    state = STATE_CHARGE;
    next_state = STATE_CHARGE;

    while(1) 
    {
        switch(state) 
        {
            case STATE_CHARGE:
                Lcd_Out(1, 11, "CHARGE");
                Lcd_Out(2, 11, "    ");

                CHRG = 1;
                TORCH = 0;

                if(!FLAG) 
                    next_state = STATE_READY;

                if(Button_Pressed(&PINA, 1, &buttonstate_a1))
                    next_state = STATE_TORCH;    

                break;

            case STATE_READY:
                Lcd_Out(2, 11, "READY");

                if(Button_Pressed(&PINA, 0, &buttonstate_a0))
                    next_state = STATE_FLASH;

                if(Button_Pressed(&PINA, 1, &buttonstate_a1))
                    next_state = STATE_TORCH;
                    
                break;

            case STATE_FLASH:
                Lcd_Out(2, 11, "     ");
                CHRG = 0;
                FLASH = 1;
                Delay_ms(600);
                FLASH = 0;

                next_state = STATE_CHARGE;
                break;

            case STATE_TORCH:
                Lcd_Out(1, 11, "TORCH ");
                Lcd_Out(2, 11, "     ");
                TORCH = 1;
                CHRG = 0;

                if(Button_Pressed(&PINA, 1, &buttonstate_a1))
                    next_state = STATE_CHARGE;
                
                break;
        }
        
        state = next_state;
        Delay_ms(POLLING_PERIOD);
    }
}

void MCU_Init() 
{
    // TORCH pin output.
    TORCH_DIR = 1;
    // CHRG pin output.
    CHRG_DIR = 1;
    // FLASH pin output.
    FLASH_DIR = 1;
    
    // FLAG pin input.
    FLAG_DIR = 0;

    // Flash button.
    DDA0_bit = 0;
    // Change mode button.
    DDA1_bit = 0;

    FLASH = 0;
    TORCH = 0;
    CHRG  = 0;

    LCD_Init();
    Lcd_Cmd(_LCD_CLEAR);
    Lcd_Cmd(_LCD_CURSOR_OFF);  

    Lcd_Out(1, 1, "Mode:");

    Delay_ms(100);
}

uint8_t Button_Pressed(unsigned int *port, unsigned int pin, uint8_t* state) 
{
    uint8_t new_state = Button(port, pin, 1, 1);
    if(*state == 0xFF && new_state == 0x00 ) 
    {
        *state = new_state;
        return 0xFF;
    }

    *state = new_state;
    return 0x00;
}

/*************** END OF FUNCTIONS *********************************************/